/*
 ķ
   Program..: _DCPATH.PRG                                           
   Author...: Roger J. Donnay                                       
   Notice...: (c) DONNAY Software Designs 1987-1994                 
   Date.....: Feb 25, 1994                                          
   Notes....: Miscellaneous path functions                          
 Ľ
*/

#include 'set.ch'
#INCLUDE "dcdialog.CH"


FUNCTION dc_path ( cFileName, lFileName )

LOCAL    nLength, nCount

cFileName := IIF(Valtype(cFileName)='C',cFileName,'')
lFileName := IIF(Valtype(lFileName)='L',lFileName,.f.)
nLength := LEN(cFilename)
IF ':'$cFilename .OR. '\'$cFilename
  FOR nCount := nLength TO 0 STEP -1
    IF SUBSTR(cFilename,nCount,1)='\' .OR. SUBSTR(cFilename,nCount,1)=':'
      EXIT
    ENDIF
  NEXT
  IF lFileName
    RETURN SUBSTR(cFilename,nCount+1)
  ENDIF
  RETURN SUBSTR(cFilename,1,nCount)
ELSE
  IF lFileName
    RETURN cFilename
  ENDIF
ENDIF
RETURN ''

// ----------------- //

FUNCTION dc_chdir ( cDirectory, lMessage )

LOCAL bErrorBlock := ErrorBlock({||_dcchdirerror(cDirectory,lMessage)}), lOk

cDirectory := IIF(VALTYPE(cDirectory)='C',ALLTRIM(cDirectory),'')
lMessage   := IIF(Valtype(lMessage)='L',lMessage,.t.)

BEGIN SEQUENCE
lOk := .f.
IF ':' $ cDirectory .AND. Len(cDirectory)=2
  CurDrive(cDirectory)
ELSE
  IF SubStr(cDirectory,1,2) # CurDrive()+':' .AND. !(':'$cDirectory)
    cDirectory := CurDrive()+':'+cDirectory
  ENDIF
  lOk := .f.
  IF Empty(cDirectory)
    CurDir()
  ENDIF
  CurDir(cDirectory)
ENDIF
lOk := .t.
END SEQUENCE
ErrorBlock(bErrorBlock)
RETURN lOk

// ----------------- //

STATIC PROCEDURE _dcchdirerror( cDirectory, lMessage )

IF lMessage
  DC_MsgBox({'Invalid Drive or Directory',cDirectory})
ENDIF
BREAK
RETURN

// ----------------- //

FUNCTION dc_curpath ( cDrive )

LOCAL cPath
cDrive := Strtran(IIF(Valtype(cDrive)='C',cDrive,''),':','')
IF Empty(cDrive)
  RETURN CurDrive() + ':' + '\' + CurDir()
ENDIF
cPath := CurDir(cDrive)
IF !Empty(cPath)
  RETURN Upper(cDrive) + ':' + '\' + CurDir()
ENDIF
RETURN ''

// ----------------- //

// dclip directory file
FUNCTION _d_c_cdfil ( cFilename )

LOCAL cEnvir

IF EMPTY(cFilename)
  RETURN .f.
ENDIF
cEnvir := ''
IF EMPTY(DC_PATH(cFileName))
  IF FExists(cFileName)
    RETURN .T.
  ENDIF
  cEnvir := DC_SETDCLIP()
ENDIF
RETURN FILE(cEnvir + cFilename)

// ----------------- //

// filename with dclip directory
FUNCTION _d_c_cdnam ( cFilename )

LOCAL cEnvir

cEnvir := ''
IF EMPTY(DC_PATH(cFileName))
  cEnvir := DC_SETDCLIP()
  IF Empty(cEnvir) .AND. FExists(cFileName)
    RETURN cFileName
  ENDIF
ENDIF
RETURN cEnvir + cFilename

// ----------------- //


FUNCTION dc_pathfound ( cFileName, cFilePath )

LOCAL nPointer, cFileDir

IF EMPTY(cFilePath)
  cFileDir := SET(_SET_DEFAULT)
  IF EMPTY(cFileDir) .AND. DC_FCLOSE(DC_SROPEN(cFileName))
    RETURN DC_CURPATH() + '\'
  ENDIF
  IF DC_FCLOSE(DC_SROPEN( cFileDir+'\'+cFileName ))
    RETURN cFileDir
  ENDIF
  cFilePath := SET(_SET_PATH)
ELSE
  cFileDir := DC_CURPATH()
  IF DC_FCLOSE(DC_SROPEN( cFileDir+'\'+cFileName ))
    RETURN cFileDir
  ENDIF
ENDIF
DO WHILE .t.
  nPointer := AT(';',cFilePath)
  cFileDir := ''
  IF nPointer > 0
    cFileDir := ALLTRIM(SUBSTR( cFilePath, 1, nPointer-1 )) + '\'
    IF DC_FCLOSE(DC_SROPEN( cFileDir+cFileName ))
      EXIT
    ENDIF
    cFilePath := SUBSTR( cFilePath, nPointer + 1 )
  ELSE
    cFileDir := ALLTRIM(cFilePath) + '\'
    IF !DC_FCLOSE(DC_SROPEN( cFileDir+cFileName ))
      cFileDir := ''
    ENDIF
    EXIT
  ENDIF
ENDDO

RETURN cFileDir

// ----------------- //

// - Open a file in shared mode - Replacement for FOPEN()

FUNCTION dc_sropen( cFileName, nMode )
cFileName := ALLTRIM(cFileName)
DC_VALTYPE( @nMode,64 )
RETURN fopen( cFileName, nMode )

* ----------

FUNCTION dc_fclose( nHandle )

IF nHandle <= 0
  RETURN .f.
ENDIF

RETURN FClose(nHandle)

// ----------------- //

FUNCTION dc_ispath( cPathString, cDirectory )

LOCAL nPointer, lFound, cFileDir

lFound := .f.
DO WHILE .t.
  nPointer := AT(';',cPathString)
  cFileDir := ''
  IF nPointer > 0
    cFileDir := ALLTRIM(SUBSTR( cPathString, 1, nPointer-1 ))
    cPathString := SUBSTR( cPathString, nPointer + 1 )
  ELSE
    cFileDir := ALLTRIM( cPathString )
  ENDIF
  IF UPPER(cDirectory) == UPPER(cFileDir)
    lFound := .t.
    EXIT
  ENDIF
  IF nPointer = 0
    EXIT
  ENDIF
ENDDO
RETURN lFound

// ----------------- //

FUNCTION dc_path2array( cPathString )

LOCAL nPointer, aPath := {}

DO WHILE .t.
  nPointer := AT(';',cPathString)
  IF nPointer > 0
    AADD( aPath, ALLTRIM(SUBSTR( cPathString, 1, nPointer-1 )))
    cPathString := SUBSTR( cPathString, nPointer + 1 )
  ELSE
    AADD( aPath, ALLTRIM( cPathString ) )
    EXIT
  ENDIF
ENDDO
RETURN aPath

// ----------------- //

FUNCTION dc_setdefault ( cPath, cMessage )

LOCAL cDefault

cMessage := IIF(Valtype(cMessage)='C',cMessage,'Invalid DEFAULT Dir:')

cPath := IIF(Valtype(cPath)='C',ALLTRIM(cPath),'')
cDefault := SET(_SET_DEFAULT)
IF !EMPTY(cPath)
  IF LEN(cPath) = 1 .AND. cPath#'\'
    cPath := cPath + ':'
  ENDIF
  IF !(RIGHT(cPath,1)$':\')
    cPath := cPath+'\'
  ENDIF
  IF !DC_FCLOSE(FCREATE(cPath+'DC_X_Y_Z.TST'))
    DC_Alert( cMessage + cPath )
    cPath := cDefault
  ELSE
    SET(_SET_DEFAULT,cPath)
  ENDIF
  ERASE (cPath+'DC_X_Y_Z.TST')
ELSE
  SET DEFAULT TO
ENDIF
RETURN SET(_SET_DEFAULT)

// --------------------- //

FUNCTION dc_isdrive ( cDir )

cDir := IIF(Valtype(cDir)='C',cDir,'')

// check for "\" passed; add if necessary
RETURN FILE(( cDir + IIF(LEFT( cDir, 1) == "\", "", "\" ) + "NUL" ) )


/*
FUNCTION dc_isdir ( cPath )

LOCAL cOldPath := DC_CurPath(), lOk := .f.

cPath := Alltrim(IIF(Valtype(cPath)='C',cPath,''))
IF Len(cPath)>3 .OR. !(':'$cPath)
  IF Right(cPath,1) = '\'
     cPath := Substr(cPath,1,Len(cPath)-1)
  ENDIF
ENDIF

DC_ChDir(cPath,.f.)
IF Upper(DC_CurPath()) == Upper(cPath)
  lOk := .t.
ELSEIF Upper(Strtran(DC_CurPath(),CurDrive()+':')) == ;
       Upper(Strtran(cPath,CurDrive()+':'))
  lOk := .t.
ENDIF
DC_ChDir(cOldPath,.f.)
RETURN lOk
*/

* ------------------

FUNCTION DC_IsDir(cDir)

cDir:=Alltrim(cDir)
IF Right(cDir,1)=='\'
cDir := Substr(cDir, 1, Len(cDir)-1)
ENDIF
RETURN fExists(cDir, "D")


* -------------------

FUNCTION DC_RunShell( cParams, cProgram )

LOCAL cPath := GetEnv('PATH')

IF Empty( DC_PathFound( cProgram, cPath ) )
  DC_WinAlert('The program: ' + cProgram + ' is not in your system PATH' + ;
              Chr(13) + ;
              'Make sure that the program exits in a directory ' + Chr(13) + ;
              'that is in your PATH statement' )
  RETURN .f.
ENDIF

RETURN RunShell( cParams, cProgram, .t. )


